<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services\FileUploaderService;
use \Datetime;

class ParamFilial extends  CrudWithParameterBag {

    const dataSourceName                 = 'vnd08600_paramfilial',
          dataSourcePadroes              = 'vnd08600_paramfilial_padroes',
	      dataSourceCaixa 	             = 'vnd08600_paramfilial_caixa',
	      dataSourceCaixa2               = 'vnd08600_paramfilial_caixa2',
          dataSourceCaixa3               = 'vnd08600_paramfilial_caixa3',
          dataSourceRegra                = 'vnd08600_paramfilial_regra',
          dataSourceConsolidacao         = 'vnd08600_paramfilial_consolidacao',
          dataSourceIntegracao           = 'vnd08600_paramfilial_integracao',
          dataSourceGestaoGerais         = 'vnd08600_paramfilial_gestao_gerais',
          dataSourceProcessamento        = 'vnd08600_paramfilial_processamento',
	      dataSourceDaypart              = 'vnd08600_paramfilial_daypart',
	      dataSourceCfeSatMfe            = 'vnd08600_paramfilial_cfeSatMfe',
	      dataSourceNfce                 = 'vnd08600_paramfilial_nfce',
          dataSourceUnidadesRelacionadas = 'unidadesRelacionadasQuery',
          dataSourceUnidadesRelacionadasPerfilAcesso = 'unidadesRelacionadasPerfilAcessoQuery',
          dataSourceProdFiliInLine       = 'vnd08600_prodFiliInLine',
          dataSourceNameParamFilial      = 'paramfilial',
          dataSourceIntegracaoContrato   = 'vnd08600_paramfilial_integracaoContrato',
          dataSourceDefNfce              = 'vnd08600_paramfilial_def_nfce',
          dataSourceEmailGerente         = 'vnd08600_paramfilial_emailGerentes',
          unidadesCliente                = 'cadConsumidor_unidadesCliente',
          dataSourceGestaoFranquias      = 'vnd08600_paramfilial_gestao_franquias';
          
    const EXTENSION_PFX = 'pfx';
    const EXTENSION_P12 = 'p12';

	protected $parameterBagColumns = array('NRORG', 'CDFILIAL'),
              $parameterBag,
              $environment,
              $entityManager,
              $dataSourceManager,
              $crudService,
              $filialService,
              $salesMessage,
              $fileUploaderService;

	
	public function __construct ($dataSourceManager, $parameterBag, $entityManager, $environment, $crudService, Services\Filial $filialService, Utilities\SalesMessage $salesMessage, FileUploaderService $fileUploaderService, $utilService, Services\Certificado $certificadoService) {
	    parent::__construct($dataSourceManager, $parameterBag);
        $this->entityManager       = $entityManager;
        $this->environment         = $environment;
        $this->crudService         = $crudService; 
        $this->filialService       = $filialService;
        $this->salesMessage        = $salesMessage;
        $this->fileUploaderService = $fileUploaderService;
        $this->utilService         = $utilService;
        $this->certificadoService  = $certificadoService;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $this->filialService->validaParavend($request);
        $conditions                = $this->utilService->getParamsAsArray($request);
        
        if($this->environment->getIdUtlPerfilFilial() == 'S'){
            $conditions['NRPERFIL'] = $this->environment->getNrPerfil();
            $this->dataSourceName = 'vnd08600_paramfilial_perfil';
            $this->parameterBagColumns = array('NRORG', 'NRPERFIL', 'CDOPERADOR');
        } else {
            $this->dataSourceName = 'vnd08600_paramfilial';
            $this->parameterBagColumns = array('NRORG', 'CDFILIAL', 'CDOPERADOR');
        }
        
        $dataSourceFilter = self::prepareParameters($conditions);
    
        
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $this->dataSourceName,
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        parent::find($newRequest, $response);
    }

    public function findPadroes(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourcePadroes;
        parent::find($request,$response);
    }

    public function findParamsFilialCaixa(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCaixa;
        parent::find($request,$response);
    }

    public function findParamsFilialCaixa2(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCaixa2;
        parent::find($request,$response);
    }

    public function findParamsFilialCaixa3(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCaixa3;
        parent::find($request,$response);
    }

    public function findRegra(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRegra;
        parent::find($request,$response);
    }

    public function findConsolidacao(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceConsolidacao;
        parent::find($request,$response);
    }

    public function findIntegracao(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceIntegracao;
        parent::find($request,$response);
    }

    public function findGestaoGerais(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceGestaoGerais;
        parent::find($request,$response);
    }

    public function findProcessamento(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceProcessamento;
        parent::find($request,$response);
    }

    public function findDaypart(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceDaypart;
        parent::find($request,$response);
    }
    
    public function findCfeSatMfe(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCfeSatMfe;
        parent::find($request,$response);
    }
    
    public function saveCfeSatMfe(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCfeSatMfe;
        self::save($request,$response);
    }
    
    public function findNfce(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceNfce;
        parent::find($request,$response);
    }

    public function findGestaoFranquias(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceGestaoFranquias;
        parent::find($request,$response);
    }
    
    public function saveNfce(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceDefNfce;
        $row = $request->getDataSet()->getRows()[0];
        
        try {
            if(isset($row['URLCERTDIGI']) && !!$row['URLCERTDIGI']) {
                $nomeArq = $row['URLCERTDIGI'][0]['name'];

                $this->fileUploaderService->setImageForSave($row, 'URLCERTDIGI', 'URLCERTDIGI');
                
                $params = array(
                    ':NRORG' => $row['NRORG']
                );
                $url = $this->filialService->getUrl($params)[0]['FILESERVERURL'];
                
                $extension = explode(".", $row['URLCERTDIGI']);
                $extension = end($extension);
               
                if(($extension !== self::EXTENSION_PFX) && ($extension !== self::EXTENSION_P12)) {
                    throw \Exception($fileName.': extensão do arquivo não aceita.');
                }
                
                //Caminho do Certificado
                $arq = $url.$row['URLCERTDIGI'];
                $pfxCertPrivado = $arq;
                $params = array(
                    ':CDFILIAL' => $row['CDFILIAL'],
                    ':NRORG' => $row['NRORG']
                );
                $senhaCertificado = $row['DSSENHACERTNFCE'];
                
                $pfxContent = file_get_contents($pfxCertPrivado);
                
                if (!openssl_pkcs12_read($pfxContent, $x509certdata, $senhaCertificado)) {
                   $response->addMessage(new Message("O certificado não pôde ser lido!<br><br>Confira se o arquivo não está corrompido ou se a senha do certificado está correta !", Message::TYPE_ERROR));
                } else {
                    
                    $CertPriv   = array();
                    $CertPriv   = openssl_x509_parse(openssl_x509_read($x509certdata['cert']));
                    $PrivateKey = $x509certdata['pkey'];
                     
                    $pub_key = openssl_pkey_get_public($x509certdata['cert']);
                    $keyData = openssl_pkey_get_details($pub_key);
                     
                    $PublicKey  = $keyData['key'];
                    $CNPJcertificado = substr($CertPriv['subject']['CN'], -14);//CNPJ
                    
                    $CNPJunidade = $this->filialService->getCNPJ($params)[0]['NRINSJURFILI'];
                    
                    if($CNPJunidade != $CNPJcertificado && false) {
                        $response->addMessage(new Message ('CNPJ do certificado não corresponde ao CNPJ da unidade !', Message::TYPE_ERROR));
                    } else {
                        $validade = date('d/m/Y', $CertPriv['validTo_time_t'] ); //Validade
                        $validade = str_replace('/', '-', $validade);
                        $NOW = date("d-m-Y");
                        $NOW = new DateTime($NOW);
                        $validade = new DateTime($validade);
                        
                        if($NOW > $validade) {
                            $response->addMessage(new Message ('O certificado está expirado !', Message::TYPE_ERROR));
                        } else {
                            $row->set('NRCERTDIGNFCE', strtoupper($CertPriv['serialNumberHex']));
                            $row->set('DTVALCERTDIGNFCE', date('d/m/Y', $CertPriv['validTo_time_t'] ));
                            $row->set('DSDIRCERTIFICADO', $arq);
                            $row->set('CDSENHACERTIF', $this->certificadoService->encrypt($senhaCertificado, 'teknisa'));
                            
                            $infoFilial = $this->certificadoService->getInfosByFilial($row['NRORG'], $row['CDFILIAL']);
                            
                            if(!empty($infoFilial['CDEMPRESA']) && !empty($infoFilial['CDINSCESTA'])) {
                                try {
                                    $dsDirCert    =  '';
                                    
                                    $cdinscesta  = $infoFilial['CDINSCESTA'];
                                    $empresa     = $infoFilial['CDEMPRESA'];
                                    $certificado = $row['URLCERTDIGI'];
                                    $nrorg       = $row['NRORG'];
                                    $nome        = $row['NMARQCERTNFCE'];
                                    $senha       = $row['CDSENHACERTIF'];
                                    $serie       = $row['NRCERTDIGNFCE'];
                                    $validade    = $row['DTVALCERTDIGNFCE'];
                                    $ambiente    = $row['IDAMBTRABNFCE'];
                                        
                                    $dsDirCert.='<'.$nomeArq.'>';
                                    $dsDirCert.=$certificado;
                                    $this->certificadoService->updateDsDirCert($cdinscesta, $empresa, $nrorg, $dsDirCert, $nome, $senha, $serie, $validade, $ambiente);
                                } catch (\Exception $e) {
                                    throw $e;
                                }
                            }
                            
                            self::save($request,$response);
                        }
                   }
                }
            } else {
                $infoFilial = $this->certificadoService->getInfosByFilial($row['NRORG'], $row['CDFILIAL']);
                            
                $cdinscesta = $infoFilial['CDINSCESTA'];
                $empresa    = $infoFilial['CDEMPRESA'];
                $nrorg      = $row['NRORG'];
                $nome       = $row['NMARQCERTNFCE'];
                $ambiente   = $row['IDAMBTRABNFCE'];
                            
                $this->certificadoService->updateDsDirCertBasico($cdinscesta, $empresa, $nrorg, $nome, $ambiente);
                
                self::save($request,$response);
            }
        } catch (\Exception $e) {
            var_dump($e->getMessage());
        }
    }

    public function saveDefault(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourcePadroes;
        $rows = $request->getDataSet()->getRows()[0];
        if(array_key_exists('VRPERCCUSTPROD', $rows) && isset($rows['VRPERCCUSTPROD'] )){
            $rows['VRPERCCUSTPROD'] = str_replace(',', '.', $rows['VRPERCCUSTPROD']);
        }
        self::save($request,$response);
    }

    public function saveCaixa(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCaixa;
        $rows = $request->getDataSet()->getRows()[0];
        if(isset($rows['NRPOSINICONS'])){
            $rows['NRPOSINICONS'] = str_replace('.', '', $rows['NRPOSINICONS']);
        }
        if(isset($rows['NRPOSFINCONS'])){
            $rows['NRPOSFINCONS'] = str_replace('.', '', $rows['NRPOSFINCONS']);
        }
        if(isset($rows['VRPEEMITROCO'] )){
            $rows['VRPEEMITROCO'] = str_replace(',', '.', $rows['VRPEEMITROCO']);
        }
        if(isset($rows['VRMINSANGRIA'] )){
            $rows['VRMINSANGRIA'] = str_replace(',', '.', $rows['VRMINSANGRIA']);
        }
        if(isset($rows['VRMAXSANGRIA'] )){
            $rows['VRMAXSANGRIA'] = str_replace(',', '.', $rows['VRMAXSANGRIA']);
        }
        self::save($request,$response);
    }

    public function saveCaixa2(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCaixa2;
        self::save($request,$response);
    }

    public function saveCaixa3(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceCaixa3;
        
        
        $rows = $request->getDataSet()->getRows()[0];
        if(!array_key_exists('IDUTCONFCADCONS', $rows)){
            $rows['IDUTCONFCADCONS'] = 'N';
        }
        if(!isset($rows['IDUTCONFCADCONS'])){
            $rows['IDUTCONFCADCONS'] = 'N';
        }
        if(isset($rows['VRMINSANGRIA']) && strlen($rows['VRMINSANGRIA']) > 6) {
            $rows['VRMINSANGRIA'] = $this->splitPointsOfString($rows['VRMINSANGRIA']);
            $rows['VRMINSANGRIA'] = str_replace(',', '.', $rows['VRMINSANGRIA']);
        }
        if(isset($rows['VRMAXSANGRIA']) && strlen($rows['VRMAXSANGRIA']) > 6) {
            $rows['VRMAXSANGRIA'] = $this->splitPointsOfString($rows['VRMAXSANGRIA']);
            $rows['VRMAXSANGRIA'] = str_replace(',', '.', $rows['VRMAXSANGRIA']);  
        }
        if(isset($rows['CDCONSUMIDOR'])){
            $rows['CDCONSUMIDPADRAO'] = $rows['CDCONSUMIDOR'];
        }
        self::save($request,$response);
    }

    public function saveRegra(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRegra;
        self::save($request,$response);
    }

    public function saveConsolidacao(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceConsolidacao;
        self::save($request,$response);
    }

    public function saveIntegracao(DTO\Request $request, DTO\Response $response){
        $this->filialService->updateIntegracaoDeliveryFilial($request->getDataSet()->getRows());
        $this->dataSourceName = self::dataSourceIntegracao;
        self::save($request,$response);
    }

    public function saveGestaoGerais(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceGestaoGerais;
        $dataSetReq = $request->getDataSet();
        $dataSetReq->setDataSourceName(self::dataSourceGestaoGerais);
        self::save($request,$response);
    }

    public function saveProcessamento(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceProcessamento;
        self::save($request,$response);
    }

    public function saveDaypart(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceDaypart;
        
        $dataSetDayPart = $this->utilService->getParamsAsArray($request);
        self::disconsiderEmptyDayParts($dataSetDayPart);
    }

    public function saveGestaoFranquias(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceGestaoFranquias;
        $dataSetReq = $request->getDataSet();
        $dataSetReq->setDataSourceName(self::dataSourceGestaoFranquias);
        self::save($request,$response);
    }
    
    private function splitPointsOfString($string) {
        $auxArray = explode('.', $string);
        $auxString = '';
        for($i = 0; $i < count($auxArray); $i++) {
            $auxString = $auxString.$auxArray[$i];    
        }
        
        return $auxString;
    }

    public function saveProdFiliInLine(DTO\Request $request, DTO\Response $response) {
        try {
            $this->dataSourceName = self::dataSourceProdFiliInLine;
            $rows = $request->getDataSet()->getRows();
            foreach($rows as $product){
                self::save($request,$response);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
	public function unidadesRelacionadas(DTO\Request $request, DTO\Response $response) {
	    $IDUTLPERFILFILIAL = $this->environment->getIdUtlPerfilFilial();
	    if($IDUTLPERFILFILIAL == 'S'){
            $this->dataSourceName =  self::dataSourceUnidadesRelacionadasPerfilAcesso;
	    } else {
	        $this->dataSourceName =  self::dataSourceUnidadesRelacionadas;
	    }
            $this->parameterBagColumns = array ("NRORG", "CDOPERADOR", "IDUTLPERFILFILIAL");
	    
        
        parent::find($request, $response);
    }


    public function alterarEstrutura(DTO\Request $request, DTO\Response $response) {
        try {
            $operador = $request->getRow();
            
            $perfil = array_key_exists('PERFIL', $operador) 
                        ? $operador['PERFIL']
                        : null;
            
            $this->alterarEstruturaService(
                $operador['CDFILIAL'], 
                $this->environment->getCdoperador(), 
                $operador['NRORG'], 
                $perfil);
            
            $row = array();
            $row['CDFILIAL'] = $operador['CDFILIAL'];
            $row['CDOPERADOR'] = $this->environment->getCdoperador();
            $row['NRORG'] = $operador['NRORG'];
            $dataSet = new DataSet('operador', array($row));
            $response->setDataSets(array($dataSet));
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function alterarEstruturaService($cdfilial, $cdoperador, $nrorg, $nrperfilselecionado) {
        $operador = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador')
            ->findOneBy(
                array(
                    'cdoperador' => $cdoperador,
                    'nrorg' => $nrorg
                )
            );
        $operador->setNrorgtrab($nrorg);
        
        $operador->setCdfilialselecionada($cdfilial);
        
        $operador->setNrorgselecionada($nrorg);
        $this->entityManager->persist($operador);
        $this->entityManager->flush();
        
        $this->environment->setCdFilial($cdfilial);
        if($this->environment->getIdUtlPerfilFilial() == 'S'){
            $this->environment->setNrPerfil($nrperfilselecionado);
            
            $this->filialService->setCdPerfilSelecionado($nrperfilselecionado, $cdoperador, $nrorg);
        }
        
    }
    
    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function buscaFiliaisCliente(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName =  self::unidadesCliente;
            $this->parameterBagColumns = array ("NRORG", "CDCLIENTE", "CDCONSUMIDOR");
            
            parent::find($request, $response);
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function buscaFiliaisConsumidor(DTO\Request $request, DTO\Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $result = $this->filialService->buscaFiliaisConsumidor($params);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetFiliaisConsumidor', $result));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function saveFiliaisConsumidor(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows();
            $params[0]['value']  = $row[0]['CDCLIENTE'];
            $params[1]['value']  = $row[0]['CDCONSUMIDOR'];
            $params[2]['value']  = $row[count($row)-1]['CDFILIAL'];//POG pra pegar sempre o ultimo 8)
            $params[3]['value']  = $row[0]['NRORG'];
            
            $this->filialService->saveFiliaisConsumidor($params);
            $message= array($this->salesMessage->getMessage('criado'));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetFiliaisConsumidor', $message));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function removeFiliaisConsumidor(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows();
            $this->filialService->removeFiliaisConsumidor($row);
            $params[0]['value']  = $row[0]['CDCLIENTE']; 
            $params[1]['value']  = $row[0]['CDCONSUMIDOR'];
            $params[2]['value']  = $row[0]['NRORG'];
            $result = $this->filialService->buscaFiliaisConsumidor($params);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('GetFiliaisConsumidor', $result));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage())); 
        }
    }
    public function findDefNfce(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceDefNfce;
        parent::find($request, $response);
    }
    
    
    public function saveDefNfce(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceDefNfce;
        self::save($request, $response); 
    }
    
    private function prepareParameters($conditions) {
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = array(
                "columnName" => $key,
                "operator"=> "=",
                "value"=> $value
            );
        }
        
        for($i = 0; $i < count($conditions); $i++) {
            if(strpos($dataSourceFilter[$i]["columnName"], '|') != false) {
                $dataSourceFilter[$i]["operator"] = "MAPPED_LIKE_ALL";
            }
        }
        
        return $dataSourceFilter;
    }
    
    public function findIntegracaoContrato(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceIntegracaoContrato;
        parent::find($request,$response);
    }
    
    public function saveIntegracaoContrato(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceIntegracaoContrato;
        self::save($request, $response); 
    }
    
    public function disconsiderEmptyDayParts($dataSetDayPart){
        empty($dataSetDayPart['ANULADAYPART1']) ? $dataSetDayPart['ANULADAYPART1'] = 'N'  :  $dataSetDayPart['ANULADAYPART1'] =  $dataSetDayPart['ANULADAYPART1'] ;
        empty($dataSetDayPart['ANULADAYPART2']) ? $dataSetDayPart['ANULADAYPART2'] = 'N'  :  $dataSetDayPart['ANULADAYPART2'] =  $dataSetDayPart['ANULADAYPART2'] ;
        empty($dataSetDayPart['ANULADAYPART3']) ? $dataSetDayPart['ANULADAYPART3'] = 'N'  :  $dataSetDayPart['ANULADAYPART3'] =  $dataSetDayPart['ANULADAYPART3'] ;
        empty($dataSetDayPart['ANULADAYPART4']) ? $dataSetDayPart['ANULADAYPART4'] = 'N'  :  $dataSetDayPart['ANULADAYPART4'] =  $dataSetDayPart['ANULADAYPART4'] ;
        if($dataSetDayPart['ANULADAYPART1'] === 'S'){
            $dataSetDayPart['HRINI1DAYPART'] = NULL;
            $dataSetDayPart['HRFIN1DAYPART'] = NULL;
        }
        if(!isset($dataSetDayPart['ANULADAYPART2']) && ($dataSetDayPart['ANULADAYPART2'] === 'S')){
            $dataSetDayPart['HRINI2DAYPART'] = NULL;
            $dataSetDayPart['HRFIN2DAYPART'] = NULL;
        }
        if(!isset($dataSetDayPart['ANULADAYPART3']) && ($dataSetDayPart['ANULADAYPART3'] === 'S')){
            $dataSetDayPart['HRINI3DAYPART'] = NULL;
            $dataSetDayPart['HRFIN3DAYPART'] = NULL;
        }
        if(!isset($dataSetDayPart['ANULADAYPART4']) && ($dataSetDayPart['ANULADAYPART4'] === 'S')){
            $dataSetDayPart['HRINI4DAYPART'] = NULL;
            $dataSetDayPart['HRFIN4DAYPART'] = NULL;
        }
        $this->filialService->saveDayPart($dataSetDayPart);
    }
    
    public function dayPartExport (DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->utilService->getParamsAsArray($request);
        $this->filialService->exportDayPart($conditions);
        
    }
    
    public function findEmailGerentes(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceEmailGerente;
        $this->parameterBagColumns = $this->parameterBagColumns;
        parent::find($request,$response);
    }
    
    public function saveEmailGerentes(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceEmailGerente;
        $row = $this->utilService->getParamsAsArray($request);
        if($this->filialService->verifyEmailCode($row)){
            self::save($request,$response);
        } else {
            $response->setError(new Error('Código já cadastrado', 500));
        }
    }
     
    public function deleteEmailGerentes(DTO\Request\Row $request, DTO\Response $response) {
        $row = $this->utilService->getParamsAsArray($request);
        $respota = $this->filialService->deleteEmailGerentes($row);
        return $respota;
    }
    
    public function updateEmailGerentes(DTO\Request\Row $request, DTO\Response $response) {
        $row = $this->utilService->getParamsAsArray($request);
        $respota = $this->filialService->updateEmailGerentes($row);
        return $respota;
    }
    
     public function saveIntegracaoSite(DTO\Request\DataSet $request, DTO\Response $response) {
         $array = array(
            "status" => '',
            'message' => ''
         );
         try {
            $dataSet = $request->getDataSet();
            $rows = $dataSet->getRows()[0];
            $respota = $this->filialService->updateIntegracaoSite($rows);
            
            if($respota){
                $array = array(
                    "status" => true,
                    'message' => 'Atualizado com sucesso'
                 );
            }else{
                $array = array(
                    "status" => false,
                    'message' => 'Nenhuma filial foi atualizada'
                 );
            }
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('resultado', $array));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
}