<?php
namespace Teknisa\MAN\Controllers;

use Teknisa\MAN\Services\TabelaDePrecoGeralImportService;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Message;
use \Setforms2\Shared\ORM\DateTime;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO; 
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities\ChunkReadFilter;

use Teknisa\MAN\Controllers\TeknisaCrud;

class TabelaDePrecoGeralImport extends TeknisaCrud {
    
    public function __construct($crudService, ParameterBag $parameterBag, TabelaDePrecoGeralImportService $service) {
        $this->service = $service;
        parent::__construct($crudService, $parameterBag);
    }
    
    public static $logMessages = array();
    
    public static function escreverLog ($data) {
        array_push(self::$logMessages, $data);
    }
    
    public static function escreverLogArray($array) {
        foreach ($array as $linha) {
            self::escreverLog($linha);
        }
    }
    
    public static function getLog () {
        return self::$logMessages;
    }
    
    public function importXlsTabGeral(DTO\Request\DataSet $request, DTO\Response $response){
       
        try {
            
            $rows = $request->getDataSet()->getRows()[0];
            $errors = '';
            $array = array(
                "path" => '',
                "file" => '',
                "status" => false,
                'message' => '',
                'nameFile' => '',
                'CDTABEPREC' => $rows['CDTABEPREC'],
                'DTINIVGPREC' => $rows['DTINIVGPREC'],
                'DTFINVGPREC' => $rows['DTFINVGPREC']
            );
            
            //extensoes permitidas
            $allowed_ext = array('xlsx', 'xls', 'ods', 'ots');
            $dirFile = "/tmp/";
            $success = '';
            
            if(!empty($rows['FILEXLS'])){
                foreach ($rows['FILEXLS'] as $key => $value) {
                    
                    $array['nameFile'] = $value['name'];
                    $ext_type = explode('.', $this->tratarNomeArquivo(utf8_decode($value['name'])));
                    
                    if(in_array($ext_type[1], $allowed_ext) === false){
                         $errors .= ' Extensão não Permitida!';
                    }
                    if($value['size'] > 1048576){
                        $errors .= ' Tamanho do arquivo deve ser até 1mb!';
                    }
                    if(empty($errors)){
                        $base64 = explode(",", $value['b64File']);
                    	$data = base64_decode($base64[1]);
                    	$fileName = uniqid().'.'.$ext_type[1];
                    	$file = $dirFile.$fileName;
                    	$array['path'] = $dirFile;
                    	$array['file'] = $fileName;
                    	//$array['filial'] = $rows['CDFILIAL'];
                    	if(file_put_contents($file, $data)){
                    	   
                    	    $array['status'] = true;
                    	    $array['message'] = 'Tipo do documento validado';
                    	     
                    	}
                    }else{
                        $array['status'] = false;
                        $array['message'] = 'Não foi possível realizar o upload. '.$errors;
                    }
                }
            }
             
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('resultado', $array));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function importarXLSTabPreco(Request $request, Response $response){
        try {
            $rows = $request->getRow();
            
            ini_set('max_execution_time', 1800);
            ini_set('memory_limit', '500M');
            $array = array(
                "path" => '',
                "file" => '',
                "status" => '',
                "message" => '',
                "controle" => 1,
                'totalSuccess' => 0,
                'totalError' => 0,
                'nameFile' => '',
                'CDTABEPREC' => $rows['CDTABEPREC'],
                'DTINIVGPREC' => $rows['DTINIVGPREC'],
                'DTFINVGPREC' => $rows['DTFINVGPREC']
            );
            
            if(!empty($rows['log'])){
                self::escreverLogArray($rows['log']);
            }else{
                self::escreverLog("## Importação Iniciada Data: ".date("d/m/Y H:i:s")." ##");
                self::escreverLog("## Arquivo importado: ".$rows['nameFile']." ##");
            }
            
            if(!empty($rows['diretorio']) && !empty($rows['file'])){
                
                $array['path'] = $rows['diretorio'];
                $array['file'] = $rows['file'];
                $array['nameFile'] = $rows['nameFile'];
                $array['stop'] = false;
                $array['totalSuccess'] = $rows['tSuccess'];
                $array['totalError'] = $rows['tError'];
                
                
                $fileName = $rows['diretorio'].$rows['file'];
                
                $objReader = \PHPExcel_IOFactory::createReaderForFile($fileName);
                
                $chunkSize = 100;
                $chunkFilter = new chunkReadFilter();
                $objReader->setReadDataOnly(true);
                $objReader->setReadFilter($chunkFilter);
                
                
                $startRow = $rows['controle'];
                $chunkFilter->setRows($startRow, $chunkSize);
                $objPHPExcel = $objReader->load($fileName);
                $sheetData = $objPHPExcel->getActiveSheet()->toArray(null,true,true,true);
                $tamanhoBloco = ($startRow + $chunkSize) - 1;
                 
                if (!empty($sheetData) && count($sheetData) >= 1) {
                    $return = $this->importarTabPrec($sheetData, $rows, $array, $startRow, $tamanhoBloco, $chunkSize);
                    self::escreverLog("TOTAL DE ERROS NA PLANILHA: " .$return['totalError'].PHP_EOL);
                    self::escreverLog($return['logRowsError'].PHP_EOL);
                    $array['status'] = $return['status'];
                    $array['message'] = $return['message'];
                    $array['log'] = $this->getLog();
                    $array['stop'] = true;
                }else{
                    self::escreverLog("Operação finalizada com sucesso! Data: ".date("d/m/Y H:i:s"));
                    self::escreverLog("Importações finalizadas com sucesso: ".$array['totalSuccess']);
                    self::escreverLog("Importações finalizadas com erros: ".$array['totalError']);
                    $array['log'] = $this->getLog();
                    $array['stop'] = true;
                    $array['status'] = true;
                    $array['message'] = "Rotina finalizada com sucesso!";
                    $array['controle'] = 1;
                    
                }
                
                $objPHPExcel->disconnectWorksheets();
                unset($objPHPExcel, $sheetData);
                 
            }else{      
                $array['status'] = false;
                $array['message'] = "Não há dados do arquivo para processar!";
               
            }
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('resultado', $array));
            
        } catch (\Exception $e) {
            self::escreverLog("## Erros Reportados na Importação ## ");
            self::escreverLog($e->getMessage());
            
            
            $array = array(
                "status" => false,
                "log" => $this->getLog(),
                'message' => "Erro ao realizar rotina!"
            );
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('resultado', $array));
        }
    }
    
    public function importarTabPrec($sheetData, $rows, $array, $startRow, $tamanhoBloco, $chunkSize){
	   
        $validation = $this->validarCabecalhoXLS($sheetData, $rows);
        
        if(!empty($validation)){
            if($validation['status']){
                
                $cabecalho = $sheetData[1];
                array_shift($sheetData);
                $arrayP = array();
                
                $arrayResult = $validation['arrayResult'];
                foreach($arrayResult as $key => $value ){
                        foreach($value as $chave => $valor ){
                            $row = $this->array_replace_key($value,$chave,$cabecalho[$chave]);
                            
                        }
                        
                        $DTINIVGPREC = explode(" ", $rows['DTINIVGPREC']);
                        $DTFINVGPREC = explode(" ", $rows['DTFINVGPREC']);
                        $value['CDTABEPREC'] = $rows['CDTABEPREC'];
                        $value['DTINIVGPREC'] = $DTINIVGPREC[0];
                        $value['DTFINVGPREC'] = $DTFINVGPREC[0];
                        
                        array_push($arrayP,$value) ;
                }
               
               unset($cabecalho);
               
               $retorno =  $this->service->saveImportarTabela($arrayP);
               if($retorno['status']){
                	$array['status'] = true;
                	$array['message'] = "##Bloco de ".$startRow." a ".$tamanhoBloco." processado...";
                	self::escreverLog("## Bloco de ".$startRow." a ".$tamanhoBloco.PHP_EOL);
                    
                    if(!empty($retorno['logRowsSucess'])){
                	    self::escreverLog(" => IMPORTAÇÃO DE PREÇOS REALIZADO COM SUCESSO".PHP_EOL);
                		self::escreverLog($retorno['message'].PHP_EOL);
                	}
                	
                	if(!empty($retorno['logRowsError'])){
                		self::escreverLog(" => IMPORTAÇÃO DE PREÇOS NÃO FOI REALIZADO".PHP_EOL);
                		self::escreverLog("Na ".$retorno['message'].PHP_EOL);
                		//self::escreverLog($retorno['logRowsError'].PHP_EOL);
                	}
                	$array['logRowsError'] = $retorno['logRowsError'];
                	$array['totalSuccess'] += $retorno['rowsSucess'];
                    $array['totalError'] += $retorno['rowsError'];
                }else{
                	$array['status'] = false;
                	$array['message'] = " => Erro ao atualizar! ". (isset($retorno['message'])) ? $retorno['message'] : '';
                }
            }else{
                $array['status'] = false;
                $array['totalError'] = $validation['rowsError'];
                $array['logRowsError'] = $validation['logRowsError']; 
                self::escreverLog(" => IMPORTAÇÅO NÃO FOI REALIZADA".PHP_EOL);
            }    
        }else{
            $array['status'] = $validation['status'];
            $array['message'] = $validation['logRowsError'];
            self::escreverLog("=> CABEÇALHO FORA DO PADRÃO PARA IMPORTAÇÃO".PHP_EOL);
        	self::escreverLog($validation['logRowsError'].PHP_EOL);
        }
        $array['controle'] = $startRow + $chunkSize;    
        $array['log'] = $this->getLog();
        return $array;
    }
    
    public function validarCabecalhoXLS($dados, $rows){
        $array = array(
            'status' => true,
            'rowsError' => 0,
            'logRowsError' => ''
        );
        if(!empty($dados)){
            $arrayCabecalho = $dados[1];
            $validarCabecalho = $this->validarCabecalho($arrayCabecalho);
            if(!$validarCabecalho){
                $array['status'] = false;
                $array['rowsError'] += 1;
                $array['logRowsError'] = 'Linha 1: Cabeçalho fora do padrão ou vazio!'.PHP_EOL;
                return $array;
            }else{
               array_shift($dados);
               array_filter($dados);
               $array = $this->validarDados($dados, $rows);
               return $array;
            }
        }else{
            $array['status'] = false;
            $array['rowsError'] += 1;
            $array['logRowsError'] = 'Sem dados para serem validados!'.PHP_EOL;
            return $array;
        }
    }
    
    public function validarDados($row, $rows){
        $intervaloHora = array('00:00-00:00');
        $intervaloDias = array('0');
        $CDPRODUTO = array(null);
        $consumidor = array(null);
        $diasConf = null;
        $arrayDias = '1_2_3_4_5_6_7_V_F'; 
        $array = array(
            'status' => true,
            'rowsError' => 0,
			'logRowsError' => '',
			'arrayResult' => null
          );
        $DTINIVGPREC = explode(" ", $rows['DTINIVGPREC']);
        $DTFINVGPREC = explode(" ", $rows['DTFINVGPREC']);
        
        $DTINIVGPREC = $DTINIVGPREC[0];
        $DTFINVGPREC = $DTFINVGPREC[0];
        
        foreach ($row as $key => $value){
            if(empty($row[$key]['A']) AND empty($row[$key]['B']) AND empty($row[$key]['C']) AND empty($row[$key]['D']) AND
               empty($row[$key]['E']) AND empty($row[$key]['F']) AND empty($row[$key]['G']) AND empty($row[$key]['H']) AND
               empty($row[$key]['I']) AND empty($row[$key]['J']) AND empty($row[$key]['K']) AND empty($row[$key]['L']) AND
               empty($row[$key]['M']) AND empty($row[$key]['N']) AND empty($row[$key]['O']) AND empty($row[$key]['P']) AND
               empty($row[$key]['Q']) AND empty($row[$key]['R']))
            {
                unset($row[$key]);
                continue;
            } 
            if(empty($row[$key]['A'])){
                $array['status'] = false;
                $array['rowsError'] += 1;
                $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna A: Coluna CDPRODUTO está vazia'. PHP_EOL;
            }else{
                if(!ctype_alnum((String)$row[$key]['A']))
                {
                    $array['status'] = false;
                    $array['rowsError'] += 1;
                    $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna A: '.$row[$key]['A'].' - Coluna CDPRODUTO está na formatação errada' . PHP_EOL;
                }
            }
            
            if($this->get_numeric($this->valorBD($row[$key]['C'])) <= 0)
            {
                $array['status'] = false;
                $array['rowsError'] += 1;
                $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna C: '.$row[$key]['C'].' - Valor informado é inválido ou vazio' . PHP_EOL;
            }
            if($this->get_numeric($this->valorBD($row[$key]['D'])) <= 0 && !empty($row[$key]['D'])){
                $array['status'] = false;
                $array['rowsError'] += 1;
                $array['logRowsError'] .= 'Linha ' . ($key + 2)  .' Coluna D: '.$row[$key]['D'].' - Valor informado é inválido ou vazio'.PHP_EOL;
            }
            if($this->get_numeric($this->valorBD($row[$key]['E'])) <= 0 && !empty($row[$key]['E']))
            {
                $array['status'] = false;
                $array['rowsError'] += 1;
                $array['logRowsError'] .= 'Linha ' . ($key + 2)  .' Coluna E: '.$row[$key]['E'].' - Valor informado é inválido ou vazio'.PHP_EOL;
            }
            if($this->get_numeric($this->valorBD($row[$key]['F'])) <= 0 && !empty($row[$key]['F']))
            {
                $array['status'] = false;
                $array['rowsError'] += 1;
                $array['logRowsError'] .= 'Linha ' . ($key + 2)  .' Coluna F: '.$row[$key]['F'].' - Valor informado é inválido ou vazio'.PHP_EOL;
            }
            
            if(!empty($row[$key]['G'])){
                if( !DateTime::createFromFormat('!H:i', $row[$key]['G']) )
                {
                    $array['status'] = false;
                    $array['rowsError'] += 1;
                    $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna G: '.$row[$key]['G'].' - Hora inválida'.PHP_EOL;
                }else{
                    $vertorTempoG = explode(':',$row[$key]['G']);
                    if((int)$vertorTempoG[0]<0 || (int)$vertorTempoG[0]>23 ||
                       (int)$vertorTempoG[1]<0 || (int)$vertorTempoG[1]>59)
                    {
                        $array['status'] = false;
                        $array['rowsError'] += 1;
                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna G: '.$row[$key]['G'].' - Hora inválida'.PHP_EOL;
                    }
                }
            } 
            
            if(!empty($row[$key]['H'])){
                if( !DateTime::createFromFormat('!H:i', $row[$key]['H']) )
                {
                    $array['status'] = false;
                    $array['rowsError'] += 1;
                    $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna H: '.$row[$key]['H'].' - Hora inválida'.PHP_EOL;
                }else{
                    $vertorTempoH= explode(':',$row[$key]['H']);
                    if((int)$vertorTempoH[0]<0 || (int)$vertorTempoH[0]>23 ||
                       (int)$vertorTempoH[1]<0 || (int)$vertorTempoH[1]>59)
                    {
                        $array['status'] = false;
                        $array['rowsError'] += 1;
                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna H: '.$row[$key]['H'].' - Hora inválida'.PHP_EOL;
                    }
                }
            }
            
            if(DateTime::createFromFormat('!H:i', $row[$key]['G']) &&
               DateTime::createFromFormat('!H:i', $row[$key]['H']))
            {
                if( (int)$vertorTempoH[0] < (int)$vertorTempoG[0])
                {
                  $array['status'] = false;
                  $array['rowsError'] += 1;
                  $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna G: '.$row[$key]['G'].' - Hora início maior que hora fim '. $row[$key]['H'].PHP_EOL;
                }
                if( (int)$vertorTempoH[0] == (int)$vertorTempoG[0] && 
                    (int)$vertorTempoH[1] <= (int)$vertorTempoG[1] )
                {
                    $array['status'] = false;
                    $array['rowsError'] += 1;
                    $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna G: '.$row[$key]['G'].' - Hora início maior que hora fim '.$row[$key]['H'].PHP_EOL;
                }   
            }
            
            if(empty($row[$key]['I']) || strlen($row[$key]['I']) != 1)
            {
                $array['status'] = false;
                $array['rowsError'] += 1;
                $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna I: Campo deve conter uma letra S - sim ou N - não'.PHP_EOL;
            }else{
                if($row[$key]['I'] == "S"){
                    $diasSemana = explode('_', $row[$key]['J']);
                    $matrizSemana = explode('_', $arrayDias);
                    if($row[$key]['J'] != "T"){
                        foreach($diasSemana as $diasLinha){
                            if(!in_array($diasLinha, $matrizSemana)) {
                                $array['status'] = false;
                                $array['rowsError'] += 1;
                                $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna J: '.$row[$key]['J'].' Campo deve conter os dias da semana, permitidos: 1_2_3_4_5_6_7_V_F ou somente T'.PHP_EOL;
                            }   
                        }
                    }
                    if(!empty($row[$key]['K'])){
                      $codExplode = explode(" - ", $row[$key]['K']);
                      if(!is_numeric((int)$codExplode[0]) || empty($codExplode[1])){
                      	$array['status'] = false;
                      	$array['rowsError'] += 1;
                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna K: Campo deve conter código e tipo do consumidor, Ex: 01 - Lojista (*Espaço - *Espaço) ou CAMPO DEVE VIR VAZIO QUANDO FOR PARA TODOS'.PHP_EOL;
                      }else {
                        $row[$key]['K'] = $codExplode[0];
                      }
                    }else {
                        $row[$key]['K'] = 'T';
                    }
                    
                    if($this->get_numeric($this->valorBD($row[$key]['L'])) <= 0){
                      $array['status'] = false;
                      $array['rowsError'] += 1;
                      $array['logRowsError'] .= 'Linha ' . ($key + 2)  .' Coluna L: '.$row[$key]['L'].' - Valor informado inválido'.PHP_EOL;
                    }
                    if($row[$key]['M'] != 'D' && $row[$key]['M'] != 'A') {
                     	$array['status'] = false;
                     	$array['rowsError'] += 1;
                     	$array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna M, Campo deve conter letra D - Desconto ou A - Acréscimo'.PHP_EOL;
                    }
                    if($row[$key]['N'] != 'P' && $row[$key]['N'] != 'V'){
                     	$array['status'] = false;
                     	$array['rowsError'] += 1;
                     	$array['logRowsError'] .= 'Linha '. ($key + 2) .' Coluna N, Campo deve conter letra P - Percentual ou V - Valor'.PHP_EOL;
                    }else{ 
                        $preco = !empty($row[$key]['C']) ? (float)$this->valorBD($row[$key]['C']) : 0.0;
                        if($row[$key]['M'] == 'D'){
    						if($row[$key]['N'] == 'P'){
    						    $valorDP = $preco*(1 - ( (float)$this->valorBD($row[$key]['L']) / 100) );
                                if($this->get_numeric($this->valorBD($row[$key]['C'])) > 0){
                                    if($valorDP <= 0 || $valorDP >= 2*$preco){
                                        $array['status'] = false;
                                        $array['rowsError'] += 1;
                                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna L: Valor de desconto maior que o valor do item'.PHP_EOL;
                                    }   
                                }
                            } 
                            if($row[$key]['N'] == 'V'){
                            	$valorDV = $preco-(float)$this->valorBD($row[$key]['L']);
                            	if($valorDV <= 0 || $valorDV >= 2*$preco){
                                  $array['status'] = false;
                                  $array['rowsError'] += 1;
                                  $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna L: Valor de desconto maior que o valor do item'.PHP_EOL;
                                }
                            }
                        }
                        if($row[$key]['M'] == 'A'){
                        	if($row[$key]['N'] == 'P'){
                            	$valorAP =   $preco*( 1 + ( (float)$this->valorBD($row[$key]['L']) / 100) );
                            	if($valorAP <= 0 || $valorAP >= 2*$preco){
                                  $array['status'] = false;
                                  $array['rowsError'] += 1;
                                  $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna L: Valor de acréscimo maior que o valor do item'.PHP_EOL;
                                }
                            }
                            if($row[$key]['N'] == 'V'){
                            	$valorAV = $preco+(float)$this->valorBD($row[$key]['L']);
                            	if( $valorAV <= 0 || $valorAV >= (2*$preco)){
                                  $array['status'] = false;
                                  $array['rowsError'] += 1;
                                  $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna L: Valor de acréscimo maior que o valor do item'.PHP_EOL;
                                }
                            }
                        }
                    }
                    if((String)$row[$key]['O'] != 'S' && (String)$row[$key]['O'] != 'N'){
                        $array['status'] = false;
                        $array['rowsError'] += 1;
                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna O, '.$row[$key]['O'].' - Campo deve conter letra S - sim ou N - não'.PHP_EOL;
                    }
                    
                  	$vertorTempoIni = explode(':',$row[$key]['P']);
                  	
                  	if(empty($row[$key]['P'])){
                  	    $array['status'] = false;
                        $array['rowsError'] += 1;
                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna P: '.$row[$key]['P'].' - Campo hora vazio'.PHP_EOL;
                  	}else if( !DateTime::createFromFormat('!H:i', $row[$key]['P']) || 
                       (int)$vertorTempoIni[0]<0 || (int)$vertorTempoIni[0]>23 ||
                       (int)$vertorTempoIni[1]<0 || (int)$vertorTempoIni[1]>59)
                    {
                        $array['status'] = false;
                        $array['rowsError'] += 1;
                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna P: '.$row[$key]['P'].' - Hora inválida'.PHP_EOL;
                    }
                    
                    $vertorTempoFim= explode(':',$row[$key]['Q']);
                    
                    if(empty($row[$key]['Q'])){
                        $array['status'] = false;
                        $array['rowsError'] += 1;
                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna Q: '.$row[$key]['Q'].' - Campo hora vazio'.PHP_EOL;
                    }else if( !DateTime::createFromFormat('!H:i', $row[$key]['Q']) || 
                       (int)$vertorTempoFim[0]<0 || (int)$vertorTempoFim[0]>23 ||
                       (int)$vertorTempoFim[1]<0 || (int)$vertorTempoFim[1]>59)
                    {
                        $array['status'] = false;
                        $array['rowsError'] += 1;
                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna Q: '.$row[$key]['Q'].' - Hora inválida'.PHP_EOL;
                    }
                    if(DateTime::createFromFormat('!H:i', $row[$key]['P']) && DateTime::createFromFormat('!H:i', $row[$key]['Q']) )
                    {
                        if( (int)$vertorTempoFim[0] < (int)$vertorTempoIni[0] )
                    	{
                          $array['status'] = false;
                          $array['rowsError'] += 1;
                          $array['logRowsError'] .= 'Linha ' . ($key + 2) .' - Campo hora início preço diferenciado maior que hora fim '.PHP_EOL;
                        }
                        
                        if( (int)$vertorTempoFim[0] == (int)$vertorTempoIni[0] &&
                            (int)$vertorTempoFim[1] < (int)$vertorTempoIni[1] ){
                            $array['status'] = false;
                            $array['rowsError'] += 1;
                            $array['logRowsError'] .= 'Linha ' . ($key + 2) .' - Campo hora início preço diferenciado maior que hora fim '.PHP_EOL;
                        }
                        
                        foreach($intervaloHora as $keyIntHora => $intHoras){
                            $vetorIni = explode('-',$intHoras);
                            $vetorHoraIni = explode(':',$vetorIni[0]);
                            $vetorHoraFim = explode(':',$vetorIni[1]);
                            
                            if( ((float)$vertorTempoIni[0] + (float)$vertorTempoIni[1]/60) <= ((float)$vetorHoraFim[0] + (float)$vetorHoraFim[1]/60) && 
                                ((float)$vertorTempoFim[0] + (float)$vertorTempoFim[1]/60) >= ((float)$vetorHoraIni[0] + (float)$vetorHoraIni[1]/60) &&
                                in_array($row[$key]['A'], $CDPRODUTO)
                            ){
                                foreach($intervaloDias as $keyIntDias => $intDias){
                                    $dias = explode('_', $intDias);
                                    if($row[$key]['J'] == "T"){
                                        $vetDias = explode('_', $arrayDias);
                                    }else $vetDias = explode('_', $row[$key]['J']);
    
                                    foreach($vetDias as $vDias){
                                        if(in_array($vDias, $dias)) {
                                            if((in_array($row[$key]['K'], $consumidor) || $row[$key]['K'] == 'T' || $consumidor == 'T'))
                                            {
                                                $valor = true;
                                                $array['status'] = false;
                                                $array['rowsError'] += 1;
                                                $diasConf += 1;   
                                            }
                                        }
                                    }
                                }
                            }else{
                                $valor = false;
                            }
                        }
                        array_push($intervaloHora, $row[$key]['P'].'-'.$row[$key]['Q']);
                        array_push($intervaloDias, $row[$key]['J']);
                        array_push($consumidor,$row[$key]['K']);  
                        array_push($CDPRODUTO, $row[$key]['A']);
                    }  
                    if($diasConf > 0){
                        $array['logRowsError'] .= 'Linha ' . ($key) .' Produto '.$row[$key]['A'].' Já existe preço diferenciado dentro do intervalo de horário informado'.PHP_EOL; //adicionar        
                    }
                    
                    $diasConf = 0;
                    $valor = false;
                    if(!empty($DTINIVGPREC) && !empty($DTFINVGPREC)){
                        if(!empty($row[$key]['R'])){
                            $vertorDataIni = explode('/',$row[$key]['R']);
                            if(!empty($vertorDataIni[2]) && is_numeric($vertorDataIni[0]) &&
                               !empty($vertorDataIni[1]) && is_numeric($vertorDataIni[1]) && (int)$vertorDataIni[1] >= 1 && (int)$vertorDataIni[1] <= 12 &&
                               !empty($vertorDataIni[0]) && is_numeric($vertorDataIni[0]) && (int)$vertorDataIni[0] >= 1 && (int)$vertorDataIni[0] <= 31
                            ){
                                if( !DateTime::createFromFormat('d/m/Y', $row[$key]['R']) ||
                                    (int)$vertorDataIni[0]<1 || (int)$vertorDataIni[0]>31 ||
                                    (int)$vertorDataIni[1]<1 || (int)$vertorDataIni[1]>12 )
                                {
                                        $array['status'] = false;
                                        $array['rowsError'] += 1;
                                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna R, Data: '.$row[$key]['R'].' - Data início inválido'.PHP_EOL;
                                }else {
                                    $dataIniVig = explode("/", $DTINIVGPREC);
                                    if((int)$vertorDataIni[2]<(int)$dataIniVig[2]){ //erro 1
                                        $array['status'] = false;
                                        $array['rowsError'] += 1;
                                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna R: '.$row[$key]['R'].' - Data fora da vigência ' .$DTINIVGPREC.PHP_EOL;
                                    }else if((int)$vertorDataIni[1]<(int)$dataIniVig[1] &&
                                             (int)$vertorDataIni[2]==(int)$dataIniVig[2]){
                                        $array['status'] = false;
                                        $array['rowsError'] += 1;
                                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna R: '.$row[$key]['R'].' - Data fora da vigência  ' .$DTINIVGPREC.PHP_EOL;
                                    }else if((int)$vertorDataIni[0]<(int)$dataIniVig[0] && 
                                             (int)$vertorDataIni[1]==(int)$dataIniVig[1] && 
                                             (int)$vertorDataIni[2]==(int)$dataIniVig[2]){
                                        $array['status'] = false;
                                        $array['rowsError'] += 1;
                                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna R: '.$row[$key]['R'].' - Data fora da vigência  ' .$DTINIVGPREC.PHP_EOL;
                                    }
                                }  
                                if(!empty($row[$key]['S'])){
                                    $vertorDataFim = explode('/',$row[$key]['S']);
                                    if(!empty($vertorDataFim[2]) && is_numeric($vertorDataFim[2]) &&
                                       !empty($vertorDataFim[1]) && is_numeric($vertorDataFim[1]) && (int)$vertorDataFim[1] >= 1 && (int)$vertorDataFim[1] <= 12 &&
                                       !empty($vertorDataFim[0]) && is_numeric($vertorDataFim[0]) && (int)$vertorDataFim[0] >= 1 && (int)$vertorDataFim[0] <= 31
                                    ){
                                        if( !DateTime::createFromFormat('d/m/Y', $row[$key]['S']) ||
                                            (int)$vertorDataFim[0]<1 || (int)$vertorDataFim[0]>31 ||
                                            (int)$vertorDataFim[1]<1 || (int)$vertorDataFim[1]>12 ){
                                            $array['status'] = false;
                                            $array['rowsError'] += 1;
                                            $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna S, Data: '.$row[$key]['S'].' - Data fim inválido'.PHP_EOL;
                                        }else{
                                            $dataFimVig = explode("/", $DTFINVGPREC);
                                            if((int)$vertorDataIni[2]>(int)$vertorDataFim[2])
                                            {
                                               $array['status'] = false;
                                               $array['rowsError'] += 1;
                                               $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna R: '.$row[$key]['R'].' - Data início maior que data fim'.PHP_EOL;
                                            }else if((int)$vertorDataIni[1]>(int)$vertorDataFim[1] && 
                                                    (int)$vertorDataIni[2]==(int)$vertorDataFim[2])
                                            {
                                                $array['status'] = false;
                                                $array['rowsError'] += 1;
                                                $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna R: '.$row[$key]['R'].' - Data início maior que data fim'.PHP_EOL;
                                            }else if((int)$vertorDataIni[0]>(int)$vertorDataFim[0] && 
                                                    (int)$vertorDataIni[1]==(int)$vertorDataFim[1] &&
                                                    (int)$vertorDataIni[2]==(int)$vertorDataFim[2])
                                            {
                                                $array['status'] = false;
                                                $array['rowsError'] += 1;
                                                $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna R: '.$row[$key]['R'].' - Data início maior que data fim'.PHP_EOL;
                                            }
                                            if((int)$vertorDataFim[2]>(int)$dataFimVig[2])
                                            {
                                                $array['status'] = false;
                                                $array['rowsError'] += 1;
                                                $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna S: '.$row[$key]['S'].' - Data fora da vigência '.$DTFINVGPREC.PHP_EOL;
                                            }else if((int)$vertorDataFim[1]>(int)$dataFimVig[1] && 
                                                     (int)$vertorDataFim[2]==(int)$dataFimVig[2])
                                            {
                                                $array['status'] = false;
                                                $array['rowsError'] += 1;
                                                $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna S: '.$row[$key]['S'].' - Data fora da vigência '.$DTFINVGPREC.PHP_EOL;
                                            }else if((int)$vertorDataFim[0]>(int)$dataFimVig[0] && 
                                                     (int)$vertorDataFim[1]==(int)$dataFimVig[1] && 
                                                     (int)$vertorDataFim[2]==(int)$dataFimVig[2])
                                            {
                                                $array['status'] = false;
                                                $array['rowsError'] += 1;
                                                $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna S: '.$row[$key]['S'].' - Data fora da vigência '.$DTFINVGPREC .PHP_EOL;
                                            }
                                        }    
                                    }else{
                                        $array['status'] = false;
                                        $array['rowsError'] += 1;
                                        $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna S: '.$row[$key]['S'].' - Data fim com formatação incorreta!'.PHP_EOL;
                                    }   
                                }else{
                                    $array['status'] = false;
                                    $array['rowsError'] += 1;
                                    $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna S: '.$row[$key]['S'].' - Data fim vazio'.PHP_EOL;
                                }
                            }else {
                                $array['status'] = false;
                                $array['rowsError'] += 1;
                                $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna R: '.$row[$key]['R'].' - Data início com formatação incorreta'.PHP_EOL;
                            }   
                        }else{
                            $array['status'] = false;
                            $array['rowsError'] += 1;
                            $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna R: '.$row[$key]['R'].' - Data início vazio'.PHP_EOL;
                        }
                        if(empty($row[$key]['S'])){
                            $array['status'] = false;
                            $array['rowsError'] += 1;
                            $array['logRowsError'] .= 'Linha ' . ($key + 2) .' Coluna S: '.$row[$key]['S'].' - Data fim vazio'.PHP_EOL;
                        }   
                    }else{
                        $array['status'] = false;
                        $array['rowsError'] += 1;
                        $array['logRowsError'] .= 'Tabela de preço sem vigência cadastrada'.PHP_EOL;
                    }
                }
            }
        }
        $array['arrayResult'] = $row;
        return $array;
    }
    
    private function validarCabecalho($arrayCabecalho){
		$retorno = false;
		$fieldsCabecalho = array(
    		'A' => 'CDPRODUTO',
    		'B' => 'NMPRODUTO', 
    		'C' => 'PRECO', 
    		'D' => 'PRECOSUG', 
    		'E' => 'PRECOSUB', 
    		'F' => 'PRECOCALC', 
    		'G' => 'HRINI', 
    		'H' => 'HRFIN', 
    		'I' => 'PRECODIF', 
    		'J' => 'NRDIASEMANPR', 
    		'K' => 'CDTIPOCONS', 
    		'L' => 'VRPRECODIA', 
    		'M' => 'IDDESCACREPR', 
    		'N' => 'IDPERVALORPR',
    		'O' => 'IDVISUACUPOM',
    		'P' => 'HRINIPRECDIA',
    		'Q' => 'HRFINPRECDIA',
    		'R' => 'DTINIVALPREC',
    		'S' => 'DTFINVALPREC',
    	
        );
       
        $arr3 = array_diff($fieldsCabecalho, $arrayCabecalho);
        
        if (count($arr3) == 0) {
            $retorno = true;
        }
		return $retorno;
	}
	
	private function array_replace_key(&$arr, $old, $new, $overwrite = true): bool {
        if (isset($arr[$new]) and !$overwrite) {
            return false;
        }
    
        $arr[$new] = $arr[$old];
        unset($arr[$old]);
    
        return true;
    }
    
    private function valorBD($valorAplicado){
		
		$posVirgula = strpos( $valorAplicado, "," );
		$posPont = strpos( $valorAplicado, "." );
		
		if (!$posVirgula === false && !$posPont === false) {
			$valor = str_replace("." , "" , $valorAplicado ); // Primeiro tira os pontos
			$valorAplicado2 = str_replace(',','.',$valor);
		}else if(!$posVirgula === false && $posPont === false){
			$valorAplicado2 = str_replace(',','.',$valorAplicado);
		}else {
			$valorAplicado2 = $valorAplicado;
		}
		
		return $valorAplicado2;
	}
	
	private function get_numeric($val) { 
        if (is_numeric($val) && $val > 0 && !empty($val)) { 
            return ($val + 0); 
        } 
        return 0; 
    } 
    
    public function tratarNomeArquivo($string){
       $partes 	= explode(".", $string);
       $extensao 	= $partes[count($partes)-1];
       $nome	= preg_replace('/\.[^.]*$/', '', $string);
       $a = 'ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõöøùúûüýýþÿŔŕ?';
       $b = 'aaaaaaaceeeeiiiidnoooooouuuuybsaaaaaaaceeeeiiiidnoooooouuuuyybyRr-';
       $nome = strtr($nome, utf8_decode($a), $b);
       $nome = str_replace(".","-",$nome);
       $nome = preg_replace( "/[^0-9a-zA-Z\.]+/",'-',$nome);
       return utf8_decode(strtolower($nome.".".$extensao));
    }
    
}